@@ -1,21 +1,23 @@
 // preferences.h
 // LiVES (lives-exe)
-// (c) G. Finch (salsaman@gmail.com) 2004 - 2016
+// (c) G. Finch (salsaman_lives@gmail.com) 2004 - 2019
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
 #ifndef HAS_LIVES_PREFS_H
 #define HAS_LIVES_PREFS_H
 
-#define PREFS_PANED_POS ((int)(200.*widget_opts.scale))
+#define PREFS_PANED_POS ((int)(300.*widget_opts.scale))
 
 // for mainw->prefs_changed
-#define PREFS_THEME_CHANGED (1<<0)
-#define PREFS_JACK_CHANGED (1<<1)
-#define PREFS_TEMPDIR_CHANGED (1<<2)
-#define PREFS_COLOURS_CHANGED (1<<3)
-#define PREFS_XCOLOURS_CHANGED (1<<4)
-#define PREFS_IMAGES_CHANGED (1<<5)
+#define PREFS_THEME_CHANGED		(1 << 0)
+#define PREFS_JACK_CHANGED		(1 << 1)
+#define PREFS_WORKDIR_CHANGED		(1 << 2)
+#define PREFS_COLOURS_CHANGED		(1 << 3)
+#define PREFS_XCOLOURS_CHANGED		(1 << 4)
+#define PREFS_IMAGES_CHANGED		(1 << 5)
+#define PREFS_MONITOR_CHANGED		(1 << 6)
+#define PREFS_NEEDS_REVERT		(1 << 7)
 
 #define PULSE_AUDIO_URL "http://www.pulseaudio.org"
 #define JACK_URL "http://jackaudio.org"
@@ -31,34 +33,35 @@ typedef struct {
 #define PB_QUALITY_MED 2  ///< default
 #define PB_QUALITY_HIGH 3
 
+  boolean pbq_adaptive;
+
   _encoder encoder; ///< from main.h
 
   short audio_player;
+#define AUD_PLAYER_NONE 0
 #define AUD_PLAYER_SOX 1
-#define AUD_PLAYER_MPLAYER 2
-#define AUD_PLAYER_JACK 3
-#define AUD_PLAYER_PULSE 4
-#define AUD_PLAYER_MPLAYER2 5
+#define AUD_PLAYER_JACK 2
+#define AUD_PLAYER_PULSE 3
 
   // string forms
+#define AUDIO_PLAYER_NONE "none"
 #define AUDIO_PLAYER_SOX "sox"
 #define AUDIO_PLAYER_JACK "jack"
-#define AUDIO_PLAYER_PULSE "pulse"
-#define AUDIO_PLAYER_PULSE_AUDIO "pulse audio" // display version
 
-#define AUDIO_PLAYER_MPLAYER "mplayer"
-#define AUDIO_PLAYER_MPLAYER2 "mplayer2"
+#define AUDIO_PLAYER_PULSE "pulse" ///< used in pref and for external players (e.g -ao pulse, -aplayer pulse)
+#define AUDIO_PLAYER_PULSE_AUDIO "pulseaudio" ///< used for display, alternate pref and alternate startup opt (-aplayer pulseaudio)
 
-
-
-  char aplayer[512]; // name, eg. "jack","pulse","sox","mplayer","mplayer2"
+  char aplayer[512]; // name, eg. "jack","pulse","sox"
 
   /// frame quantisation type
   short q_type;
 #define Q_FILL 1
 #define Q_SMOOTH 1
 
-  char tmpdir[PATH_MAX];  ///< kept in locale encoding
+  char workdir[PATH_MAX];  ///< kept in locale encoding
+
+  char configfile[PATH_MAX];  ///< kept in locale encoding (config settings) [default ~/.local/config/lives)
+  char config_datadir[PATH_MAX];  ///< kept in locale encoding (general config files) (default ~/.local/share/lives)
 
   // utf8 encoding
   char def_vid_load_dir[PATH_MAX];
@@ -71,63 +74,103 @@ typedef struct {
   char prefix_dir[PATH_MAX];
   char lib_dir[PATH_MAX];
 
-
+  char image_type[16];
   char image_ext[16];
 
-  uint32_t warning_mask;
+  uint64_t warning_mask;
+
+  /// bits 10, 11, 13, 18 and 19 set (off by default)
+  // (should have been done by reversing the sense of these bits, but it is too late now
+#define DEF_WARNING_MASK 0x000C2C04ul
+
   // if these bits are set, we do not show the warning
-#define WARN_MASK_FPS (1<<0)
-#define WARN_MASK_FSIZE (1<<1)
+#define WARN_MASK_FPS 	       					(1ul << 0)
+#define WARN_MASK_FSIZE 	       				(1ul << 1)
+#define WARN_MASK_UNUSED1ul 	       				(1ul << 2)  ///< was "save_quality"
+#define WARN_MASK_SAVE_SET 		       			(1ul << 3)
+#define WARN_MASK_NO_MPLAYER 			       		(1ul << 4)
+#define WARN_MASK_RENDERED_FX 					(1ul << 5)
+#define WARN_MASK_NO_ENCODERS 					(1ul << 6)
+#define WARN_MASK_LAYOUT_MISSING_CLIPS 				(1ul << 7)
+#define WARN_MASK_LAYOUT_CLOSE_FILE 				(1ul << 8)
+#define WARN_MASK_LAYOUT_DELETE_FRAMES 				(1ul << 9)
 
-  /// no longer used
-#define WARN_MASK_SAVE_QUALITY (1<<2)
+  /** off by default on a fresh install */
+#define WARN_MASK_LAYOUT_SHIFT_FRAMES 				(1ul << 10)
 
-#define WARN_MASK_SAVE_SET (1<<3)
-#define WARN_MASK_NO_MPLAYER (1<<4)
-#define WARN_MASK_RENDERED_FX (1<<5)
-#define WARN_MASK_NO_ENCODERS (1<<6)
-#define WARN_MASK_LAYOUT_MISSING_CLIPS (1<<7)
-#define WARN_MASK_LAYOUT_CLOSE_FILE (1<<8)
-#define WARN_MASK_LAYOUT_DELETE_FRAMES (1<<9)
+  /** off by default on a fresh install */
+#define WARN_MASK_LAYOUT_ALTER_FRAMES 				(1ul << 11)
+#define WARN_MASK_DUPLICATE_SET        				(1ul << 12)
 
-  /* next two are off by default (on a fresh install) */
-#define WARN_MASK_LAYOUT_SHIFT_FRAMES (1<<10)
-#define WARN_MASK_LAYOUT_ALTER_FRAMES (1<<11)
+  /** off by default on a fresh install */
+#define WARN_MASK_EXIT_MT 		       			(1ul << 13)
+#define WARN_MASK_DISCARD_SET 					(1ul << 14)
+#define WARN_MASK_AFTER_DVGRAB 					(1ul << 15)
+#define WARN_MASK_MT_ACHANS 			       		(1ul << 16)
+#define WARN_MASK_LAYOUT_DELETE_AUDIO 				(1ul << 17)
 
-#define WARN_MASK_DUPLICATE_SET (1<<12)
+  /** off by default on a fresh install */
+#define WARN_MASK_LAYOUT_SHIFT_AUDIO 				(1ul << 18)
 
-#define WARN_MASK_EXIT_MT (1<<13)
-#define WARN_MASK_DISCARD_SET (1<<14)
-#define WARN_MASK_AFTER_DVGRAB (1<<15)
+  /** off by default on a fresh install */
+#define WARN_MASK_LAYOUT_ALTER_AUDIO 				(1ul << 19)
 
-#define WARN_MASK_MT_ACHANS (1<<16)
+#define WARN_MASK_MT_NO_JACK 			       		(1ul << 20)
+#define WARN_MASK_OPEN_YUV4M 					(1ul << 21)
+#define WARN_MASK_MT_BACKUP_SPACE 				(1ul << 22)
+#define WARN_MASK_LAYOUT_POPUP 					(1ul << 23)
+#define WARN_MASK_CLEAN_AFTER_CRASH 				(1ul << 24)
+#define WARN_MASK_NO_PULSE_CONNECT 				(1ul << 25)
+#define WARN_MASK_LAYOUT_WIPE 					(1ul << 26)
+#define WARN_MASK_LAYOUT_GAMMA 					(1ul << 27)
+#define WARN_MASK_VJMODE_ENTER 					(1ul << 28)
+#define WARN_MASK_CLEAN_INVALID 		       		(1ul << 29)
+#define WARN_MASK_LAYOUT_LB	 		       		(1ul << 30)
 
-#define WARN_MASK_LAYOUT_DELETE_AUDIO (1<<17)
+  // reserved (on / unset by default)
+#define WARN_MASK_RSVD_16					(1ul << 31)
+#define WARN_MASK_RSVD_15					(1ul << 32)
+#define WARN_MASK_RSVD_14					(1ul << 33)
+#define WARN_MASK_RSVD_13					(1ul << 34)
+#define WARN_MASK_RSVD_12					(1ul << 35)
+#define WARN_MASK_RSVD_11					(1ul << 36)
+#define WARN_MASK_RSVD_10					(1ul << 37)
+#define WARN_MASK_RSVD_9					(1ul << 38)
+#define WARN_MASK_RSVD_8					(1ul << 39)
+#define WARN_MASK_RSVD_7					(1ul << 40)
+#define WARN_MASK_RSVD_6					(1ul << 41)
+#define WARN_MASK_RSVD_5					(1ul << 42)
+#define WARN_MASK_RSVD_4					(1ul << 43)
+#define WARN_MASK_RSVD_3					(1ul << 44)
+#define WARN_MASK_RSVD_2					(1ul << 45)
+#define WARN_MASK_RSVD_1					(1ul << 46)
+#define WARN_MASK_RSVD_0					(1ul << 47)
 
-  /* next two are off by default (on a fresh install) */
-#define WARN_MASK_LAYOUT_SHIFT_AUDIO (1<<18)
-#define WARN_MASK_LAYOUT_ALTER_AUDIO (1<<19)
+  // for bits 48 - 63, the sense will be reversed, in case we need anything else off
+  // by default
+#define WARN_MASK_RSVD_OFF_15					(1ul << 48)
+#define WARN_MASK_RSVD_OFF_14					(1ul << 49)
+#define WARN_MASK_RSVD_OFF_13					(1ul << 50)
+#define WARN_MASK_RSVD_OFF_12					(1ul << 51)
+#define WARN_MASK_RSVD_OFF_11					(1ul << 52)
+#define WARN_MASK_RSVD_OFF_10					(1ul << 53)
+#define WARN_MASK_RSVD_OFF_9					(1ul << 54)
+#define WARN_MASK_RSVD_OFF_8					(1ul << 55)
+#define WARN_MASK_RSVD_OFF_7					(1ul << 56)
+#define WARN_MASK_RSVD_OFF_6					(1ul << 57)
+#define WARN_MASK_RSVD_OFF_5					(1ul << 58)
+#define WARN_MASK_RSVD_OFF_4					(1ul << 59)
+#define WARN_MASK_RSVD_OFF_3					(1ul << 60)
+#define WARN_MASK_RSVD_OFF_2					(1ul << 61)
+#define WARN_MASK_RSVD_OFF_1					(1ul << 62)
+#define WARN_MASK_RSVD_OFF_0					(1ul << 63)
 
-#define WARN_MASK_MT_NO_JACK (1<<20)
-
-#define WARN_MASK_OPEN_YUV4M (1<<21)
-
-#define WARN_MASK_MT_BACKUP_SPACE (1<<22)
-
-#define WARN_MASK_LAYOUT_POPUP (1<<23)
-
-#define WARN_MASK_CLEAN_AFTER_CRASH (1<<24)
-
-#define WARN_MASK_NO_PULSE_CONNECT (1<<25)
-
-#define WARN_MASK_LAYOUT_WIPE (1<<26)
   char cmd_log[PATH_MAX];
-  char effect_command[PATH_MAX*2];
-  char video_open_command[PATH_MAX*2];
-  char audio_play_command[PATH_MAX*2];
+  char effect_command[PATH_MAX * 2];
+  char video_open_command[PATH_MAX * 2];
+  char audio_play_command[PATH_MAX * 2];
   char cdplay_device[PATH_MAX];  ///< locale encoding
   double default_fps;
-  int bar_height;
   boolean pause_effect_during_preview;
   boolean open_decorated;
   int sleep_time;
@@ -145,20 +188,18 @@ typedef struct {
 #define SEPWIN_TYPE_STICKY 1
 
   boolean show_player_stats;
-  boolean show_framecount;
+  //  boolean show_framecount; - use hide_framebar
   boolean show_subtitles;
   boolean loop_recording;
   boolean discard_tv;
   boolean save_directories;
-  boolean safer_preview;
   int rec_opts;
-#define REC_FRAMES (1<<0)
-#define REC_FPS (1<<1)
-#define REC_EFFECTS (1<<2)
-#define REC_CLIPS (1<<3)
-#define REC_AUDIO (1<<4)
-#define REC_AFTER_PB (1<<5)
-
+#define REC_FRAMES	(1 << 0)
+#define REC_FPS		(1 << 1)
+#define REC_EFFECTS	(1 << 2)
+#define REC_CLIPS	(1 << 3)
+#define REC_AUDIO	(1 << 4)
+#define REC_AFTER_PB	(1 << 5)
 
   int audio_src;
 #define AUDIO_SRC_INT 0
@@ -171,26 +212,34 @@ typedef struct {
   boolean omc_noisy; ///< send success/fail
   boolean omc_events; ///< send other events
 
-  short startup_phase; ///< -1 = fresh install, 1 = tmpdir set, 2, pre-audio start, 3, pre-tests, 100 = all tests passed
-  char *wm; ///<window manager name
+  /// 0 = normal , -1 or 1: fresh install, 2: workdir set, 3: startup tests passed, 4: aud pl chosen, 5: pre interface seln., 100 setup complete
+  short startup_phase;
   int ocp; ///< open_compression_percent : get/set in prefs
 
   boolean antialias;
 
-  boolean ignore_tiny_fps_diffs;
+  double fps_tolerance;
 
   short rte_keys_virtual;
 
+  boolean show_msg_area;
+
+  // values for trickplay - TODO: add to prefs dialog
+  double blendchange_amount;
+  int scratchfwd_amount, scratchback_amount;
+  double fpschange_amount;
+
   uint32_t jack_opts;
-#define JACK_OPTS_TRANSPORT_CLIENT (1<<0)   ///< jack can start/stop
-#define JACK_OPTS_TRANSPORT_MASTER (1<<1)  ///< transport master
-#define JACK_OPTS_START_TSERVER (1<<2)     ///< start transport server
-#define JACK_OPTS_NOPLAY_WHEN_PAUSED (1<<3) ///< play audio even when transport paused
-#define JACK_OPTS_START_ASERVER (1<<4)     ///< start audio server
-#define JACK_OPTS_TIMEBASE_START (1<<5)    ///< jack sets play start position
-#define JACK_OPTS_TIMEBASE_CLIENT (1<<6)    ///< full timebase client
-#define JACK_OPTS_TIMEBASE_MASTER (1<<7)   ///< timebase master (not implemented yet)
-#define JACK_OPTS_NO_READ_AUTOCON (1<<8)   ///< do not auto connect read clients when playing ext audio
+#define JACK_OPTS_TRANSPORT_CLIENT	(1 << 0)   ///< jack can start/stop
+#define JACK_OPTS_TRANSPORT_MASTER	(1 << 1)  ///< transport master
+#define JACK_OPTS_START_TSERVER		(1 << 2)     ///< start transport server
+#define JACK_OPTS_NOPLAY_WHEN_PAUSED	(1 << 3) ///< play audio even when transport paused
+#define JACK_OPTS_START_ASERVER		(1 << 4)     ///< start audio server
+#define JACK_OPTS_TIMEBASE_START	(1 << 5)    ///< jack sets play start position
+#define JACK_OPTS_TIMEBASE_CLIENT	(1 << 6)    ///< full timebase client
+#define JACK_OPTS_TIMEBASE_MASTER	(1 << 7)   ///< timebase master (not implemented yet)
+#define JACK_OPTS_NO_READ_AUTOCON	(1 << 8)   ///< do not auto con. rd clients when playing ext aud
+#define JACK_OPTS_TIMEBASE_LSTART	(1 << 9)    ///< LiVES sets play start position
 
   char jack_tserver[PATH_MAX];
   char jack_aserver[PATH_MAX];
@@ -202,9 +251,9 @@ typedef struct {
   LiVESList *acodec_list;
   int acodec_list_to_format[AUDIO_CODEC_NONE];
 
-  uint32_t audio_opts;
-#define AUDIO_OPTS_FOLLOW_CLIPS (1<<0)
-#define AUDIO_OPTS_FOLLOW_FPS (1<<1)
+  volatile uint32_t audio_opts;
+#define AUDIO_OPTS_FOLLOW_CLIPS		(1 << 0)
+#define AUDIO_OPTS_FOLLOW_FPS		(1 << 1)
 
   boolean event_window_show_frame_events;
   boolean crash_recovery; ///< TRUE==maintain mainw->recovery file
@@ -213,29 +262,26 @@ typedef struct {
 
   boolean move_effects;
 
+#define DEF_MT_UNDO_SIZE 32 ///< MB
+
   int mt_undo_buf;
   boolean mt_enter_prompt;
 
-  int mt_def_width;
-  int mt_def_height;
+  int mt_def_width, mt_def_height;
   double mt_def_fps;
 
-  int mt_def_arate;
-  int mt_def_achans;
-  int mt_def_asamps;
-  int mt_def_signed_endian;
+  int mt_def_arate, mt_def_achans, mt_def_asamps, mt_def_signed_endian;
 
   boolean mt_exit_render;
   boolean render_prompt;
-
+  boolean mt_show_ctx;
   boolean mt_pertrack_audio;
   int mt_backaudio;
 
-  int mt_auto_back; ///< time diff to backup (-1 == never, 0 == after every change, >0 == seconds)
-
-  boolean ar_clipset;
-  boolean ar_layout;
+  int mt_auto_back; ///< time diff to backup (-1 == never, 0 == after every change, > 0 == seconds)
 
+  /// auto-reload
+  boolean ar_clipset, ar_layout;
   char ar_clipset_name[128]; ///< locale
   char ar_layout_name[PATH_MAX];  ///< locale
 
@@ -247,12 +293,10 @@ typedef struct {
 
   boolean osc_start;
 
-  int virt_height; ///< n screens vert.
-
   boolean concat_images;
 
   boolean render_audio;
-  boolean normalise_audio;
+  boolean normalise_audio;  ///< for future use
 
   boolean instant_open;
   boolean auto_deint;
@@ -263,12 +307,14 @@ typedef struct {
 
   boolean force_single_monitor;
 
+  boolean show_urgency_msgs;
+  boolean show_overlay_msgs;
+  boolean render_overlay;
+
+  /// deprecated
   int midi_check_rate;
   int midi_rpt;
 
-#define OMC_DEV_MIDI 1<<0
-#define OMC_DEV_JS 1<<1
-#define OMC_DEV_FORCE_RAW_MIDI 1<<2
   uint32_t omc_dev_opts;
 
   char omc_js_fname[PATH_MAX];  ///< utf8
@@ -282,8 +328,11 @@ typedef struct {
 
 #ifdef ALSA_MIDI
   boolean use_alsa_midi;
+  boolean alsa_midi_dummy;
 #endif
 
+  int midi_rcv_channel;
+
   int startup_interface;
 
 #define STARTUP_CE 0
@@ -291,13 +340,12 @@ typedef struct {
 
   boolean ce_maxspect;
 
-  boolean show_button_icons;
-
   boolean lamp_buttons;
 
   boolean autoload_subs;
 
-  int rec_stop_gb;
+#define DEF_REC_STOP_GB 10.
+  double rec_stop_gb;
 
   int max_modes_per_key; ///< maximum effect modes per key
 
@@ -312,6 +360,7 @@ typedef struct {
   boolean stream_audio_out;
   boolean unstable_fx;
   boolean letterbox; ///< playback with letterbox
+  boolean letterbox_mt; ///< playback with letterbox (multitrack)
   boolean enc_letterbox; ///< encode with letterbox
 
   boolean force_system_clock; /// < force system clock (rather than soundcard) for timing ( better for high framerates )
@@ -319,27 +368,36 @@ typedef struct {
   boolean force64bit;
 
   boolean auto_trim_audio;
+  boolean keep_all_audio;
 
   /** default 0; 1==use old (bad) behaviour on bigendian machines (r/w bigend ints/doubles); 2==bad reads, good writes */
   int bigendbug;
 
-
-  // these are defualt values; actual values can be adjusted in Preferences
-#define DEF_DS_WARN_LEVEL 250000000  // 250MB
+  // these are default values; actual values can be adjusted in Preferences
+#define DEF_DS_WARN_LEVEL 2500000000  // 2.5 GB
   uint64_t ds_warn_level; ///< diskspace warn level bytes
-#define DEF_DS_CRIT_LEVEL 20000000 // 20MB
+#define DEF_DS_CRIT_LEVEL 250000000 // 250MB
   uint64_t ds_crit_level; ///< diskspace critical level bytes
 
+#define DEF_DISK_QUOTA 50  /// def 50 GiB
+  uint64_t disk_quota; /// (SOFT LIMIT) max space we can use for all our files (0 means unlimited (up to ds_crtical, HARD LIMIT))
 
-#define LIVES_CDISK_LEAVE_ORPHAN_SETS (1<<0)
-#define LIVES_CDISK_LEAVE_BFILES (1<<1)
-#define LIVES_CDISK_REMOVE_ORPHAN_LAYOUTS (1<<2)
-#define LIVES_CDISK_LEAVE_MARKER_FILES (1<<3)
-#define LIVES_CDISK_LEAVE_MISC_FILES (1<<4)
+#define DEF_MSG_TEXTSIZE 4 // LIVES_FONTSIZE_LARGE (via lives_textsize_to_string())
+#define DEF_MAX_MSGS 10000
 
-#define LIVES_CDISK_REMOVE_LOCK_FILES (1<<5) ///< not yet implemented - TODO
-#define LIVES_CDISK_REBUILD_ORDER_FILES (1<<6) ///< not yet implemented - TODO
+#define LIVES_CDISK_LEAVE_ORPHAN_SETS		(1 << 0)
+#define LIVES_CDISK_LEAVE_BFILES		(1 << 1)
+#define LIVES_CDISK_REMOVE_ORPHAN_LAYOUTS	(1 << 2)
+#define LIVES_CDISK_LEAVE_MARKER_FILES		(1 << 3)
+#define LIVES_CDISK_LEAVE_MISC_FILES		(1 << 4)
+#define LIVES_CDISK_REMOVE_ORPHAN_CLIPS		(1 << 5) /// ! == recover
+#define LIVES_CDISK_REMOVE_STALE_RECOVERY	(1 << 6)
+#define LIVES_CDISK_LEAVE_EMPTY_DIRS		(1 << 7)
 
+#define LIVES_CDISK_REMOVE_LOCK_FILES		(1 << 16) ///< not yet implemented - TODO
+#define LIVES_CDISK_REBUILD_ORDER_FILES		(1 << 17) ///< not yet implemented - TODO
+
+#define LIVES_CDISK_SEND_TO_TRASH		(1 << 31)
 
   uint32_t clear_disk_opts;
 
@@ -349,12 +407,13 @@ typedef struct {
 
   LiVESList *disabled_decoders;
 
-  char backend_sync[PATH_MAX];
-  char backend[PATH_MAX];
+  char backend_sync[PATH_MAX * 4];
+  char backend[PATH_MAX * 4];
 
   char weed_plugin_path[PATH_MAX];
   char frei0r_path[PATH_MAX];
   char ladspa_path[PATH_MAX];
+  char libvis_path[PATH_MAX];
 
   boolean present;
 
@@ -372,6 +431,70 @@ typedef struct {
 
   boolean mt_load_fuzzy;
 
+  boolean hide_framebar;
+
+  boolean hfbwnp;
+
+  boolean show_asrc;
+
+  float ahold_threshold;
+
+  double screen_gamma;
+
+  int max_messages;
+  int msg_textsize;
+
+  double screen_scale;
+
+  boolean load_rfx_builtin;
+
+  boolean apply_gamma;
+  boolean use_screen_gamma;
+  boolean btgamma; ///< allows clips to be *stored* with bt709 gamma - CAUTION not backwards compatible, untested
+
+  boolean show_tooltips;
+
+  float volume; ///< audio volume level (for jack and pulse)
+
+  boolean vj_mode; // optimise for VJing (in progress, experimental)
+
+  boolean allow_easing;
+
+  boolean show_dev_opts;
+  boolean dev_show_dabg;
+  boolean dev_show_timing;
+
+  boolean msgs_pbdis;
+
+  boolean noframedrop;
+
+  boolean back_compat;
+
+  char pa_start_opts[255];
+  boolean pa_restart;
+
+  boolean cb_is_switch;
+
+  boolean interactive;
+  boolean extra_colours;
+  boolean pref_trash; ///< user prefers trash to delete
+  boolean autoclean; ///< remove temp files on shutdown / startup
+
+  boolean show_desktop_panel;
+  boolean show_msgs_on_startup;
+  boolean show_disk_quota;
+  double quota_limit;
+
+  boolean rr_crash;
+  int rr_qmode;
+  boolean rr_super;
+  boolean rr_pre_smooth;
+  boolean rr_qsmooth;
+  int rr_fstate;
+  boolean rr_amicro;
+  boolean rr_ramicro;
+
+  char def_author[1024]; ///< TODO - add to prefs windo
 } _prefs;
 
 enum {
@@ -398,20 +521,23 @@ enum {
   N_COLUMNS
 };
 
+#define PREFWIN_WIDTH (GUI_SCREEN_WIDTH * .9)
+#define PREFWIN_HEIGHT (GUI_SCREEN_HEIGHT * .9)
 
-#define PREFWIN_WIDTH (mainw->scr_width*.9)
-#define PREFWIN_HEIGHT (mainw->scr_height*.9)
-
-#define DS_WARN_CRIT_MAX 1000000. ///< MB.
+#define DS_WARN_CRIT_MAX 1000000. ///< MB. (default 1 TB)
 
 /// prefs window
 typedef struct {
   ulong encoder_ofmt_fn;
   ulong encoder_name_fn;
+  ulong close_func;
+  LiVESAccelGroup *accel_group;
+
   LiVESWidget *prefs_dialog;
 
   LiVESWidget *prefs_list;
   LiVESWidget *prefs_table;
+  LiVESWidget *tlabel;
   LiVESWidget *vbox_right_gui;
   LiVESWidget *vbox_right_multitrack;
   LiVESWidget *vbox_right_decoding;
@@ -441,7 +567,7 @@ typedef struct {
   LiVESWidget *scrollw_right_jack;
   LiVESWidget *scrollw_right_midi;
   LiVESWidget *right_shown;
-  LiVESWidget *cancelbutton;
+  LiVESWidget *revertbutton;
   LiVESWidget *applybutton;
   LiVESWidget *closebutton;
   LiVESWidget *stop_screensaver_check;
@@ -450,6 +576,10 @@ typedef struct {
   LiVESWidget *mouse_scroll;
   LiVESWidget *fs_max_check;
   LiVESWidget *recent_check;
+  LiVESWidget *checkbutton_lb; //< letterbox
+  LiVESWidget *checkbutton_lbmt;
+  LiVESWidget *checkbutton_screengamma;
+  LiVESWidget *spinbutton_gamma;
   LiVESWidget *video_open_entry;
   LiVESWidget *audio_command_entry;
   LiVESWidget *vid_load_dir_entry;
@@ -457,24 +587,39 @@ typedef struct {
   LiVESWidget *audio_dir_entry;
   LiVESWidget *image_dir_entry;
   LiVESWidget *proj_dir_entry;
-  LiVESWidget *tmpdir_entry;
+  LiVESWidget *workdir_entry;
   LiVESWidget *cdplay_entry;
   LiVESWidget *spinbutton_def_fps;
   LiVESWidget *pbq_combo;
+  LiVESWidget *pbq_adaptive;
   LiVESWidget *ofmt_combo;
   LiVESWidget *audp_combo;
+  LiVESWidget *pa_gens;
   LiVESWidget *rframes;
   LiVESWidget *rfps;
   LiVESWidget *rclips;
   LiVESWidget *reffects;
   LiVESWidget *raudio;
   LiVESWidget *rextaudio;
+  LiVESWidget *rintaudio;
   LiVESWidget *rdesk_audio;
+  LiVESWidget *rr_crash;
+  LiVESWidget *rr_super;
+  LiVESWidget *rr_combo;
+  LiVESWidget *rr_pre_smooth;
+  LiVESWidget *rr_qsmooth;
+  LiVESWidget *rr_scombo;
+  LiVESWidget *rr_amicro;
+  LiVESWidget *rr_ramicro;
   LiVESWidget *encoder_combo;
+  LiVESWidget *checkbutton_load_rfx;
+  LiVESWidget *checkbutton_apply_gamma;
   LiVESWidget *checkbutton_antialias;
   LiVESWidget *checkbutton_threads;
   LiVESWidget *spinbutton_warn_ds;
   LiVESWidget *spinbutton_crit_ds;
+  LiVESWidget *dsl_label;
+  LiVESWidget *dsc_label;
   LiVESWidget *checkbutton_warn_fps;
   LiVESWidget *checkbutton_warn_mplayer;
   LiVESWidget *checkbutton_warn_save_set;
@@ -494,6 +639,9 @@ typedef struct {
   LiVESWidget *checkbutton_warn_after_dvgrab;
   LiVESWidget *checkbutton_warn_no_pulse;
   LiVESWidget *checkbutton_warn_layout_wipe;
+  LiVESWidget *checkbutton_warn_layout_gamma;
+  LiVESWidget *checkbutton_warn_layout_lb;
+  LiVESWidget *checkbutton_warn_vjmode_enter;
   LiVESWidget *checkbutton_show_stats;
   LiVESWidget *checkbutton_warn_fsize;
   LiVESWidget *checkbutton_warn_mt_achans;
@@ -501,6 +649,7 @@ typedef struct {
   LiVESWidget *checkbutton_warn_yuv4m_open;
   LiVESWidget *checkbutton_warn_mt_backup_space;
   LiVESWidget *checkbutton_warn_after_crash;
+  LiVESWidget *checkbutton_warn_invalid_clip;
   LiVESWidget *spinbutton_warn_fsize;
   LiVESWidget *spinbutton_bwidth;
   LiVESWidget *theme_combo;
@@ -510,6 +659,8 @@ typedef struct {
   LiVESWidget *cbutton_mab;
   LiVESWidget *cbutton_infot;
   LiVESWidget *cbutton_infob;
+  LiVESWidget *fb_filebutton;
+  LiVESWidget *se_filebutton;
   LiVESWidget *theme_style2;
   LiVESWidget *theme_style3;
   LiVESWidget *theme_style4;
@@ -531,6 +682,10 @@ typedef struct {
   LiVESWidget *jpeg;
   LiVESWidget *mt_enter_prompt;
   LiVESWidget *spinbutton_ocp;
+  LiVESWidget *nmessages_spin;
+  LiVESWidget *msgs_unlimited;
+  LiVESWidget *msgs_pbdis;
+  LiVESWidget *msg_textsize_combo;
   LiVESWidget *acodec_combo;
   LiVESWidget *spinbutton_osc_udp;
   LiVESWidget *spinbutton_rte_keys;
@@ -542,13 +697,16 @@ typedef struct {
   LiVESWidget *checkbutton_jack_master;
   LiVESWidget *checkbutton_jack_client;
   LiVESWidget *checkbutton_jack_tb_start;
+  LiVESWidget *checkbutton_jack_mtb_start;
   LiVESWidget *checkbutton_jack_tb_client;
   LiVESWidget *checkbutton_jack_pwp;
   LiVESWidget *checkbutton_jack_read_autocon;
   LiVESWidget *checkbutton_start_tjack;
   LiVESWidget *checkbutton_start_ajack;
+  LiVESWidget *checkbutton_parestart;
   LiVESWidget *checkbutton_afollow;
   LiVESWidget *checkbutton_aclips;
+  LiVESWidget *spinbutton_ext_aud_thresh;
   LiVESWidget *spinbutton_mt_def_width;
   LiVESWidget *spinbutton_mt_def_height;
   LiVESWidget *spinbutton_mt_def_fps;
@@ -571,8 +729,14 @@ typedef struct {
   LiVESWidget *checkbutton_auto_trim;
   LiVESWidget *checkbutton_nobord;
   LiVESWidget *checkbutton_concat_images;
+  LiVESWidget *checkbutton_show_asrc;
+  LiVESWidget *checkbutton_show_ttips;
+  LiVESWidget *checkbutton_hfbwnp;
   LiVESWidget *forcesmon;
   LiVESWidget *forcesmon_hbox;
+  LiVESWidget *cb_show_msgstart;
+  LiVESWidget *cb_show_quota;
+  LiVESWidget *cb_autoclean;
   LiVESList *pbq_list;
   char *audp_name;
   char *orig_audp_name;
@@ -583,63 +747,101 @@ typedef struct {
   LiVESWidget *omc_midi_entry;
   LiVESWidget *spinbutton_midicr;
   LiVESWidget *spinbutton_midirpt;
+  LiVESWidget *midichan_combo;
   LiVESWidget *alsa_midi;
+  LiVESWidget *alsa_midi_dummy;
   LiVESWidget *button_midid;
   LiVESWidget *rb_startup_ce;
   LiVESWidget *rb_startup_mt;
   LiVESWidget *jack_int_label;
   LiVESWidget *checkbutton_ce_maxspect;
   LiVESWidget *checkbutton_button_icons;
-  LiVESWidget *temp_label;
+  LiVESWidget *checkbutton_extra_colours;
+  LiVESWidget *workdir_label;
   LiVESWidget *checkbutton_stream_audio;
   LiVESWidget *checkbutton_rec_after_pb;
   LiVESWidget *wpp_entry;
   LiVESWidget *frei0r_entry;
   LiVESWidget *ladspa_entry;
+  LiVESWidget *libvis_entry;
   LiVESWidget *cdda_hbox;
   LiVESWidget *midi_hbox;
   LiVESWidget *frameblank_entry;
   LiVESWidget *sepimg_entry;
+  LiVESWidget *def_author_entry;
+  LiVESWidget *dialog_hpaned;
   LiVESTreeSelection *selection;
+
+  boolean ignore_apply; ///< dont light the apply button when thing changes (for external calls), normally FALSE
   boolean needs_restart;
 } _prefsw;
 
 /// startup overrides from commandline
 typedef struct {
   boolean ign_clipset;
+  boolean ign_layout;
   boolean ign_osc;
+  boolean ign_jackopts;
   boolean ign_aplayer;
+  boolean ign_asource;
   boolean ign_stmode;
   boolean ign_vppdefs;
+  boolean ign_vjmode;
+  boolean ign_dscrit;
+  boolean ign_configfile;
+  boolean ign_config_datadir;
 } _ign_opts;
 
 typedef struct {
-  char tmpdir[PATH_MAX];
+  // if a pref also has an entry in future_prefs, be wary of changing its value
+  // seek to understand why it has a variant value
+
+  char workdir[PATH_MAX];
   char theme[64];
   char vpp_name[64]; ///< new video playback plugin
+
   int vpp_fixed_fps_numer;
   int vpp_fixed_fps_denom;
+
   double vpp_fixed_fpsd;
+
   int vpp_palette;
   int vpp_YUV_clamping;
+
   int vpp_fwidth;
   int vpp_fheight;
+
   int vpp_argc;
-  char **vpp_argv;
-
-  _encoder encoder;
-  boolean show_recent;
-  boolean show_tool;
-  boolean osc_start;
-  int startup_interface;
-  uint32_t jack_opts;
-
   int nfx_threads;
 
+  char **vpp_argv;
+  uint64_t disk_quota;
+
+  _encoder encoder;
+
+  boolean show_recent;
+
+  boolean osc_start;
+  int startup_interface;
+
+  uint32_t jack_opts;
+  int audio_src;
+
+  uint32_t audio_opts;
+  short pb_quality;
+  short sepwin_type;
 
   LiVESList *disabled_decoders;
   LiVESList *disabled_decoders_new;
 
+  volatile float volume; ///< audio volume level (for jack and pulse)
+
+  boolean vj_mode;
+  boolean ar_clipset;
+
+  int msg_textsize;
+  boolean pref_trash; ///< user prefers trash to delete (future / present swapped)
+  boolean letterbox_mt;
 } _future_prefs;
 
 _prefs *prefs;
@@ -649,59 +851,63 @@ _prefsw *prefsw;
 void set_acodec_list_from_allowed(_prefsw *, render_details *);
 void  rdet_acodec_changed(LiVESCombo *acodec_combo, livespointer user_data);
 
-_prefsw *create_prefs_dialog(void);
-
-boolean on_prefs_delete_event(LiVESWidget *, LiVESXEvent *, livespointer prefsw);
-
-void on_preferences_activate(LiVESMenuItem *, livespointer);
-
-void on_prefs_close_clicked(LiVESButton *, livespointer);
-
-void on_prefs_revert_clicked(LiVESButton *, livespointer);
-
-void on_prefs_apply_clicked(LiVESButton *, livespointer user_data);
-
 void set_vpp(boolean set_in_prefs);
 
-void on_prefDomainChanged(LiVESTreeSelection *, livespointer);
-
+_prefsw *create_prefs_dialog(LiVESWidget *saved_dialog);
+boolean on_prefs_delete_event(LiVESWidget *, LiVESXEvent *, livespointer prefsw);
+void on_preferences_activate(LiVESMenuItem *, livespointer);
+void on_prefs_close_clicked(LiVESButton *, livespointer);
+void on_prefs_revert_clicked(LiVESButton *, livespointer);
+void on_prefs_apply_clicked(LiVESButton *, livespointer user_data);
+void on_prefs_page_changed(LiVESTreeSelection *, _prefsw *);
 void populate_combo_box(LiVESCombo *, LiVESList *data);
-
 void set_combo_box_active_string(LiVESCombo *, char *active_str);
 
-void prefsw_set_astream_settings(_vid_playback_plugin *);
-void prefsw_set_rec_after_settings(_vid_playback_plugin *);
+void prefsw_set_astream_settings(_vid_playback_plugin *, _prefsw *);
+void prefsw_set_rec_after_settings(_vid_playback_plugin *, _prefsw *);
 
 void pref_change_images(void);
 void pref_change_xcolours(void);
 void pref_change_colours(void);
 
-
 void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 
+// TODO:
+/*typedef struct {
+  const char *pref_name;
+  int type;
+  } lives_preference;
 
-// factories (pseudo prefs), called from cpp binding
+  const lives_preference [] = {
+  {PREF_REC_EXT_AUDIO, WEED_SEED_BOOL},
+  };
+
+  then:
+
+  widget = lives_standard_widget_for_pref(const char *prefname, const char *label, val, min, max, step, page, dp, box, rb_group_or_combo_list, tooltip);
+*/
+
+// NOTE: the following definitions must match with equivalent keys in smogrify
 
 #define PREF_REC_EXT_AUDIO "rec_ext_audio"
 #define PREF_AUDIO_OPTS "audio_opts"
-#define PREF_SEPWIN_STICKY "sepwin_sticky"
+#define PREF_SEPWIN_TYPE "sepwin_type"
 #define PREF_MT_EXIT_RENDER "mt_exit_render"
 
-
+// factories non-cpp
+#define PREF_SHOW_ASRC "show_audio_src"
+#define PREF_HFBWNP "hide_framebar_when_not_playing"
 
 // normal prefs
 
 /////////////////// string values
 
-#define PREF_WORKING_DIR "tempdir"
+#define PREF_WORKING_DIR "workdir"
+#define PREF_WORKING_DIR_OLD "tempdir"
 #define PREF_PREFIX_DIR "prefix_dir" // readonly
 #define PREF_LIB_DIR "lib_dir" // readonly
 
-#define PREF_SESSION_TEMPDIR "session_tempdir"
-
 #define PREF_AUDIO_PLAYER "audio_player"
-#define PREF_AUDIO_PLAY_COMMAND "audio_play_command"
-
 #define PREF_AUDIO_SRC "audio_src"
 
 #define PREF_MONITORS "monitors"
@@ -709,10 +915,11 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_LADSPA_PATH "ladspa_path"
 #define PREF_WEED_PLUGIN_PATH "weed_plugin_path"
 #define PREF_FREI0R_PATH "frei0r_path"
+#define PREF_LIBVISUAL_PATH "libvis_path"
 
 #define PREF_VID_PLAYBACK_PLUGIN "vid_playback_plugin"
 
-#define PREF_DEFAULT_IMAGE_FORMAT "default_image_format"
+#define PREF_DEFAULT_IMAGE_TYPE "default_image_format"
 
 #define PREF_VIDEO_OPEN_COMMAND "video_open_command"
 
@@ -726,9 +933,12 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_AR_LAYOUT "ar_layout"
 #define PREF_AR_CLIPSET "ar_clipset"
 
-#define PREF_CURRENT_AUTOTRANS "current_autotrans"
-#define PREF_DEF_AUTOTRANS "def_autotrans"  // readonly
+#define PREF_ACTIVE_AUTOTRANS "active_autotrans"
 
+#define PREF_SCREEN_SCALE "screen_scale"
+#define PREF_PASTARTOPTS "pa_start_opts"
+
+#define PREF_DEF_AUTHOR "default_author_name"
 
 ////////////////////// utf8 values
 
@@ -743,16 +953,12 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_VID_SAVE_DIR "vid_save_dir"
 #define PREF_VID_LOAD_DIR "vid_load_dir"
 
-#define PREF_RECENT1 "recent1"
-#define PREF_RECENT2 "recent2"
-#define PREF_RECENT3 "recent3"
-#define PREF_RECENT4 "recent4"
-
+#define PREF_RECENT "recent"
 
 /////////////////// integer64 values
 #define PREF_DS_WARN_LEVEL "ds_warn_level"
 #define PREF_DS_CRIT_LEVEL "ds_crit_level"
-
+#define PREF_DISK_QUOTA "disk_quota"
 
 /////////////////// integer32 values
 #define PREF_STARTUP_PHASE "startup_phase"
@@ -768,10 +974,20 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 
 #define PREF_NFX_THREADS "nfx_threads"
 
+#define PREF_BTGAMMA "experimental_bt709_gamma"
+#define PREF_USE_SCREEN_GAMMA "use_screen_gamma"
+#define PREF_SCREEN_GAMMA "screen_gamma"
+
 #define PREF_CLEAR_DISK_OPTS "clear_disk_opts"
 
 #define PREF_MAX_DISP_VTRACKS "max_disp_vtracks"
 
+#define PREF_MAX_MSGS "max_text_messages"
+#define PREF_MSG_TEXTSIZE "msg_textsize"
+#define PREF_MSG_PBDIS "msg_disable_during_playback"
+
+#define PREF_NOFRAMEDROP "no_framedrop"
+
 #define PREF_RTE_KEYS_VIRTUAL "rte_keys_virtual"
 
 #define PREF_JACK_OPTS "jack_opts"
@@ -779,9 +995,9 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_MIDI_CHECK_RATE "midi_check_rate"
 #define PREF_MIDI_RPT "midi_rpt"
 
-#define PREF_ENCODER_ACODEC "encoder_acodec"
+#define PREF_MIDI_RCV_CHANNEL "midi_rcv_channel"
 
-#define PREF_AUDIO_OPTS "audio_opts"
+#define PREF_ENCODER_ACODEC "encoder_acodec"
 
 #define PREF_RECORD_OPTS "record_opts"
 
@@ -799,11 +1015,16 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_MT_UNDO_BUF "mt_undo_buf"
 
 #define PREF_MT_BACKAUDIO "mt_backaudio"
-
+#define PREF_MT_SHOW_CTX "mt_show_ctx"
 #define PREF_WARN_FILE_SIZE "warn_file_size"
 
 #define PREF_DL_BANDWIDTH_K "dl_bandwidth_K"
 
+#define PREF_SCFWD_AMOUNT "trickplay_scratch_fwd"
+#define PREF_SCBACK_AMOUNT "trickplay_scratch_back"
+
+#define PREF_RRQMODE "recrender_quant_mode"
+#define PREF_RRFSTATE "recrender_fx_posn_state"
 
 ////////// boolean values
 #define PREF_SHOW_RECENT_FILES "show_recent_files"
@@ -818,10 +1039,12 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_CE_MAXSPECT "ce_maxspect"
 #define PREF_OPEN_MAXIMISED "open_maximised"
 #define PREF_AUTO_TRIM_PAD_AUDIO "auto_trim_pad_audio"
+#define PREF_KEEP_ALL_AUDIO "never_trim_audio"
 #define PREF_MOUSE_SCROLL_CLIPS "mouse_scroll_clips"
 #define PREF_SHOW_BUTTON_ICONS "show_button_icons"
 #define PREF_STREAM_AUDIO_OUT "stream_audio_out"
 #define PREF_CE_THUMB_MODE "ce_thumb_mode"
+#define PREF_LOAD_RFX_BUILTIN "load_rfx_builtin"
 #define PREF_ANTIALIAS "antialias"
 #define PREF_FILESEL_MAXIMISED "filesel_maximised"
 #define PREF_SHOW_PLAYER_STATS "show_player_stats"
@@ -834,65 +1057,104 @@ void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 #define PREF_CONCAT_IMAGES "concat_images"
 #define PREF_SAVE_DIRECTORIES "save_directories"
 #define PREF_CONSERVE_SPACE "conserve_space"
+#define PREF_PUSH_AUDIO_TO_GENS "push_audio_to_gens"
+#define PREF_APPLY_GAMMA "apply_gamma"
+#define PREF_SHOW_TOOLTIPS "show_tooltips"
+#define PREF_SHOW_URGENCY "show_urgency_messages"
+#define PREF_SHOW_OVERLAY_MSGS "show_overlay_messages"
+#define PREF_UNSTABLE_FX "allow_unstable_effects"
+#define PREF_ALLOW_EASING "allow_easing"
+#define PREF_SHOW_DEVOPTS "show_developer_options"
+#define PREF_VJMODE "vj_mode_startup"
+#define PREF_LETTERBOX "letterbox_ce"
+#define PREF_LETTERBOXMT "letterbox_mt"
+#define PREF_PARESTART "pa_restart"
+#define PREF_PBQ_ADAPTIVE "pb_quality_adaptive"
+#define PREF_EXTRA_COLOURS "extra_colours" /// add to prefs window
+#define PREF_SHOW_SUBS "show_subtitles" /// add to prefs window
+#define PREF_AUTOLOAD_SUBS "autoload_subtitles" /// add to prefs window
+#define PREF_AUTOCLEAN_TRASH "autoclean_trash" ///< remove unneeded files on shutdown / startup
+#define PREF_PREF_TRASH "prefer_trash" ///< prefer trash to delete
+#define PREF_MSG_START "show_msgs_on_startup" /// pop up msgs box on startup
+#define PREF_SHOW_QUOTA "show_quota_on_startup" /// pop up quota box on startup
 
+#define PREF_RRCRASH "recrender_crash_protection" /// option for rendering recordings
+#define PREF_RRSUPER "recrender_super" /// option for rendering recordings
+#define PREF_RRPRESMOOTH "recrender_presmooth" /// option for rendering recordings
+#define PREF_RRQSMOOTH "recrender_qsmooth" /// option for rendering recordings
+#define PREF_RRAMICRO "recrender_amicro" /// option for rendering recordings
+#define PREF_RRRAMICRO "recrender_rend_amicro" /// option for rendering recordings
 
+#define PREF_BACK_COMPAT "backwards_compatibility" ///< forces backwards compatibility with earlier versions
 
 ////////// double values
 #define PREF_MT_DEF_FPS "mt_def_fps"
-
 #define PREF_DEFAULT_FPS "default_fps"
 
+#define PREF_BLEND_AMOUNT "trickplay_blend_change"
+#define PREF_FPSCHANGE_AMOUNT "trickplay_fpschange"
 
+///////// float values
+#define PREF_AHOLD_THRESHOLD "ahold_threshold"
+#define PREF_MASTER_VOLUME "master_volume"
 
 ////////// list values
 #define PREF_DISABLED_DECODERS "disabled_decoders"
 
+boolean pref_factory_bool(const char *prefidx, boolean newval, boolean permanent);
+boolean pref_factory_string(const char *prefidx, const char *newval, boolean permanent);
+boolean pref_factory_int(const char *prefidx, int newval, boolean permanent);
+boolean pref_factory_int64(const char *prefidx, int64_t newval, boolean permanent);
+boolean pref_factory_float(const char *prefidx, float newval, boolean permanent);
+boolean pref_factory_bitmapped(const char *prefidx, int bitfield, boolean newval, boolean permanent);
+boolean pref_factory_string_choice(const char *prefidx, LiVESList *list, const char *strval, boolean permanent);
 
+boolean has_pref(const char *key);
 
-// defaults (readonly)
-/////////////// string values
-#define PREF_SOX_COMMAND "sox_command"
-#define PREF_MPLAYER_AUDIO_COMMAND "mplayer_audio_command"
-#define PREF_MPLAYER2_AUDIO_COMMAND "mplayer2_audio_command"
+LiVESResponseType get_pref_from_file(const char *filename, const char *key, char *val, int maxlen);
 
-
-
-void pref_factory_bool(const char *prefidx, boolean newval);
-void pref_factory_int(const char *prefidx, int newval);
-void pref_factory_bitmapped(const char *prefidx, int bitfield, boolean newval);
-
-
-int get_pref(const char *key, char *val, int maxlen);
-int get_pref_from_file(const char *filename, const char *key, char *val, int maxlen);
-int get_pref_utf8(const char *key, char *val, int maxlen);
-void get_pref_default(const char *key, char *val, int maxlen);
+int get_utf8_pref(const char *key, char *val, int maxlen);
+LiVESResponseType get_string_pref(const char *key, char *val, int maxlen);
+LiVESResponseType get_string_prefd(const char *key, char *val, int maxlen, const char *def);
 boolean get_boolean_pref(const char *key);
 double get_double_pref(const char *key);
+double get_double_prefd(const char *key, double defval);
 int get_int_pref(const char *key);
 LiVESList *get_list_pref(const char *key);
 boolean get_colour_pref(const char *key, lives_colRGBA64_t *lcol);
-boolean get_theme_colour_pref(const char *themefile, const char *key, lives_colRGBA64_t *lcol);
-void set_pref(const char *key, const char *value);
-void set_pref_utf8(const char *key, const char *value);
-void delete_pref(const char *key);
-void set_boolean_pref(const char *key, boolean value);
-void set_double_pref(const char *key, double value);
-void set_int_pref(const char *key, int value);
-void set_int64_pref(const char *key, int64_t value);
-void set_list_pref(const char *key, LiVESList *values);
-void set_colour_pref(const char *key, lives_colRGBA64_t *lcol);
+boolean get_theme_colour_pref(const char *key, lives_colRGBA64_t *lcol);
+
+boolean get_boolean_prefd(const char *key, boolean defval);
+int get_int_prefd(const char *key, int defval);
+int64_t get_int64_prefd(const char *key, int64_t defval);
+
+int delete_pref(const char *key);
+
+int set_string_pref(const char *key, const char *value);
+int set_string_pref_priority(const char *key, const char *value);
+int set_utf8_pref(const char *key, const char *value);
+int set_boolean_pref(const char *key, boolean value);
+int set_double_pref(const char *key, double value);
+int set_int_pref(const char *key, int value);
+int set_int64_pref(const char *key, int64_t value);
+int set_list_pref(const char *key, LiVESList *values);
+int set_colour_pref(const char *key, lives_colRGBA64_t *lcol);
 void set_theme_pref(const char *themefile, const char *key, const char *value);
 void set_theme_colour_pref(const char *themefile, const char *key, lives_colRGBA64_t *lcol);
+
 boolean apply_prefs(boolean skip_warnings);
 void save_future_prefs(void);
 
-void set_palette_prefs(void);
+void set_palette_prefs(boolean save);
+
+void toggle_sets_pref(LiVESWidget *widget, livespointer prefidx);
 
 // permissions
 
+#define LIVES_PERM_INVALID 0
 #define LIVES_PERM_OSC_PORTS 1
+#define LIVES_PERM_DOWNLOAD_LOCAL 2
+#define LIVES_PERM_COPY_LOCAL 3
 
-boolean lives_ask_permission(int what);
-
-
+boolean lives_ask_permission(char **argv, int argc, int offs);
 #endif
